/*
 * @(#)51    1.3  src/pdwebrte/azn/eaidemo/EAIDemo/JavaSource/com/tivoli/am/eb/web/eaidemo/AbstractEAIDemoServlet.java, pdweb.ws.authn.eai, amwebrte700  8/2/05  11:27:43
 * ===========================================================================
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2005 All Rights Reserved.
 * ===========================================================================
 *
 */
package com.tivoli.am.eb.web.eaidemo;


import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.tivoli.am.eb.web.localresponsedemo.LocalResponseRedirectConstants;


/**
 * An abstract servlet that performs EAI operations common to both the user ID
 * and PAC operations.
 *
 * @author Tivoli Systems
 */
public abstract class AbstractEAIDemoServlet extends HttpServlet {
    /**
     * The name of the header used to communicate the user id entered in the
     * login form.
     */
    public static final String ORIG_USER_ATTR_NAME = "eai-orig-user";
    protected EAIDemoServerConfig config = null;
    private String declaredAuthLevel = null;
    private String declaredTamUser = null;


    /**
     * Variable information that could have come from the login/reauth/stepup
     * form.
     */
    private String requestedUrl = null;


    /**
     * Returns the authentication level the user declared in the login page.
     *
     * @return the auth level as a string, null if it wasn't submitted.
     */
    public String getDeclaredAuthLevel() {
        return declaredAuthLevel;
    }


    /**
     * Returns the user name that was inputted in the login page.
     *
     * @return the user name as a string, null if it wasn't submitted.
     */
    public String getDeclaredTamUser() {
        return declaredTamUser;
    }


    /**
     * Returns the url that was initially requested. This will only work if the
     * url was sent in a local response redirect request.
     *
     * @return the url of the initial request, null if not available.
     */
    public String getRequestedUrl() {
        return requestedUrl;
    }


    /**
     * Responds to a POST operation by adding the EAI headers and printing a
     * simple message to the user. The message will not be seen unless WebSEAL
     * is misconfigured (WebSEAL will either redirect the client to another
     * page or display its own login success page).
     *
     * @param req The HTTP request object
     * @param resp The HTTP response object
     *
     * @throws ServletException if the servlet cannot process the request
     * @throws IOException if the servlet encounters a problem writing to the
     *         client
     */
    public void doPost(HttpServletRequest req, HttpServletResponse resp)
                throws ServletException, IOException
    {
        getOperationData(req);

        Object principal = null;

        try {
            principal = doAuthentication(req);
        } catch(Exception ex) {
            throw new ServletException(ex);
        }

        resp.setContentType("text/html");
        resp.setStatus(HttpServletResponse.SC_OK);

        doEAIHeaders(principal, resp);
        doCommonEAIHeaders(req, resp);

        // add a cookie for testing purposes
        Cookie cookie = new Cookie("TESTCOOKIE", "TESTVALUE");

        cookie.setPath("/");
        resp.addCookie(cookie);

        PrintWriter out = resp.getWriter();

        out.flush();
        out.println("");
        out.println("<HTML><BODY>If you can read this text, you've either " +
                    "misconfigured EAI or you're not going through a " +
                    "WebSEAL junction.</BODY></HTML>");
        out.println("");

        out.flush();
        out.close();
    }


    /**
     * Sets up common servlet configuration.
     *
     * @throws ServletException if an error occured which causes the servlet to
     *         initialize incorrectly.
     */
    public void init() throws ServletException {
        super.init();

        // parse and check the config
        config = EAIDemoServerConfig.getInstance(this.getServletContext());
    }


    /**
     * Finds the authentication data in the client's request and converts it to
     * a principal appropriate to the type of EAI operation (user ID or PAC).
     *
     * @param req The HTTP request from the client.
     *
     * @return A principal for inclusion in the HTTP headers of the response.
     *         The object returned from this method will be passed into
     *         doEAIHeaders().
     */
    protected abstract Object doAuthentication(HttpServletRequest req)
                                        throws Exception;


    /**
     * Sets the appropriate headers and respective values based on the
     * configuration of the servlet and EAI type (user ID or PAC). The header
     * names returned can be configured by editing the servlet context
     * parameters found in the file <code>WEB-INF/web.xml</code>.
     *
     * @param principal The object returned from doAuthentication(). It is
     *        assumed that the developer knows the data type and can cast down
     *        appropriately.
     * @param resp The HTTP response where the headers will be sent.
     */
    protected abstract void doEAIHeaders(Object principal,
                                         HttpServletResponse resp);


    /**
     * Analyzes the request for hidden form fields coming from the login form.
     * That information is set in the instance variables for the servlet, and
     * can be used by any servlet extending this one.
     *
     * @param req request handle.
     */
    private void getOperationData(HttpServletRequest req) {
        // look for the TAM username
        declaredTamUser = req.getParameter(LocalResponseRedirectConstants.MACRO_USERNAME);

        // look for the requested URL
        requestedUrl = req.getParameter(LocalResponseRedirectConstants.MACRO_URL);

        // look for the auth level specified first by the user, then by WebSEAL
        declaredAuthLevel = req.getParameter("authnlevelfield");

        if(declaredAuthLevel == null) {
            declaredAuthLevel = req.getParameter(LocalResponseRedirectConstants.MACRO_AUTHNLEVEL);
        }
    }


    /**
     * Performs any header additions common to both user ID and PAC EAI types.
     *
     * @param req The HTTP request that was submitted to this servlet.
     * @param resp The HTTP response that will be sent to the client.
     */
    private void doCommonEAIHeaders(HttpServletRequest req,
                                    HttpServletResponse resp) {
        // set EAI redirect URL header
        if(config.getEAIRedirURLHeader() != null) {
            if((config.getDefaultRedirectURL() != null) &&
                          (config.getDefaultRedirectURL().length() > 0)) {
                resp.setHeader(config.getEAIRedirURLHeader(),
                               config.getDefaultRedirectURL());
            }
        }
    }
}
